@@ -13,14 +13,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import math
 from datetime import datetime
 from hashlib import blake2b
 
-from scalecodec.utils.ss58 import ss58_decode_account_index
+from scalecodec.utils.ss58 import ss58_decode_account_index, ss58_decode, ss58_encode, is_valid_ss58_address
 
 from scalecodec.base import ScaleType, ScaleBytes
-from scalecodec.exceptions import InvalidScaleTypeValueException
+from scalecodec.exceptions import InvalidScaleTypeValueException, MetadataCallFunctionNotFound
 from scalecodec.utils.math import trailing_zeros, next_power_of_two
 
 
@@ -33,9 +33,6 @@ class Compact(ScaleType):
 
     def process_compact_bytes(self):
         compact_byte = self.get_next_bytes(1)
-        if compact_byte == b'':
-            self.compact_bytes = compact_byte
-            return self.compact_bytes
         try:
             byte_mod = compact_byte[0] % 4
         except IndexError:
@@ -65,7 +62,9 @@ class Compact(ScaleType):
 
         if self.sub_type:
 
-            byte_data = self.get_decoder_class(self.sub_type, ScaleBytes(self.compact_bytes)).process()
+            byte_data = self.get_decoder_class(
+                type_string=self.sub_type, data=ScaleBytes(self.compact_bytes), runtime_config=self.runtime_config
+            ).process()
 
             if type(byte_data) is int and self.compact_length <= 4:
                 return int(byte_data / 4)
@@ -146,7 +145,7 @@ class Option(ScaleType):
     def process_encode(self, value):
 
         if value is not None and self.sub_type:
-            sub_type_obj = self.get_decoder_class(self.sub_type)
+            sub_type_obj = self.get_decoder_class(self.sub_type, runtime_config=self.runtime_config)
             return ScaleBytes('0x01') + sub_type_obj.encode(value)
 
         return ScaleBytes('0x00')
@@ -194,6 +193,14 @@ class OptionBytes(ScaleType):
 
         return None
 
+    def process_encode(self, value):
+
+        if value is not None:
+            sub_type_obj = Bytes()
+            return ScaleBytes('0x01') + sub_type_obj.encode(value)
+
+        return ScaleBytes('0x00')
+
 
 # TODO replace in metadata
 class String(ScaleType):
@@ -311,6 +318,19 @@ class U128(ScaleType):
             raise ValueError('{} out of range for u128'.format(value))
 
 
+class U256(ScaleType):
+
+    def process(self):
+        return int(int.from_bytes(self.get_next_bytes(32), byteorder='little'))
+
+    def process_encode(self, value):
+
+        if 0 <= int(value) <= 2**256 - 1:
+            return ScaleBytes(bytearray(int(value).to_bytes(32, 'little')))
+        else:
+            raise ValueError('{} out of range for u256'.format(value))
+
+
 class I8(ScaleType):
 
     def process(self):
@@ -424,7 +444,7 @@ class H512(ScaleType):
 
 class Struct(ScaleType):
 
-    def __init__(self, data, type_mapping=None, **kwargs):
+    def __init__(self, data=None, type_mapping=None, **kwargs):
 
         if type_mapping:
             self.type_mapping = type_mapping
@@ -445,23 +465,53 @@ class Struct(ScaleType):
     def process_encode(self, value):
         data = ScaleBytes(bytearray())
 
-        if type(value) is list:
+        for key, data_type in self.type_mapping:
+            if key not in value:
+                raise ValueError('Element "{}" of struct is missing in given value'.format(key))
+
+            element_obj = self.get_decoder_class(
+                type_string=data_type, metadata=self.metadata, runtime_config=self.runtime_config
+            )
+            data += element_obj.encode(value[key])
+
+        return data
+
+
+class Tuple(ScaleType):
+    def __init__(self, data=None, type_mapping=None, **kwargs):
+
+        if type_mapping:
+            self.type_mapping = type_mapping
+
+        super().__init__(data, **kwargs)
+
+    def process(self):
+
+        result = ()
+
+        for member_type in self.type_mapping:
+            if member_type is None:
+                member_type = 'Null'
+            result += (self.process_type(member_type, metadata=self.metadata).value,)
+
+        return result
+
+    def process_encode(self, value):
+        data = ScaleBytes(bytearray())
+
+        if type(value) is list or type(value) is tuple:
             if len(value) != len(self.type_mapping):
-                raise ValueError('Element count of value ({}) doesn\'t match type_mapping ({})'.format(len(value), len(self.type_mapping)))
+                raise ValueError('Element count of value ({}) doesn\'t match type_definition ({})'.format(
+                    len(value), len(self.type_mapping))
+                )
 
-            for idx, (key, data_type) in enumerate(self.type_mapping):
+            for idx, member_type in enumerate(self.type_mapping):
 
-                element_obj = self.get_decoder_class(data_type, metadata=self.metadata)
+                element_obj = self.get_decoder_class(
+                    member_type, metadata=self.metadata, runtime_config=self.runtime_config
+                )
                 data += element_obj.encode(value[idx])
 
-        else:
-            for key, data_type in self.type_mapping:
-                if key not in value:
-                    raise ValueError('Element "{}" of struct is missing in given value'.format(key))
-
-                element_obj = self.get_decoder_class(data_type, metadata=self.metadata)
-                data += element_obj.encode(value[key])
-
         return data
 
 
@@ -496,7 +546,7 @@ class Set(ScaleType):
             if item in value:
                 result += set_mask
 
-        u64_obj = self.get_decoder_class(self.value_type)
+        u64_obj = self.get_decoder_class(type_string=self.value_type, runtime_config=self.runtime_config)
 
         return u64_obj.encode(result)
 
@@ -511,10 +561,10 @@ class Era(ScaleType):
     in which it is valid.
     """
 
-    def __init__(self, data=None, sub_type=None, metadata=None):
+    def __init__(self, **kwargs):
         self.period = None
         self.phase = None
-        super().__init__(data, sub_type, metadata)
+        super().__init__(**kwargs)
 
     def process(self):
 
@@ -629,76 +679,6 @@ class CompactMoment(CompactU32):
         return self.value.isoformat()
 
 
-class BoxProposal(ScaleType):
-    type_string = 'Box<Proposal>'
-
-    def __init__(self, data, **kwargs):
-        self.call_index = None
-        self.call_function = None
-        self.call_module = None
-        self.call_args = []
-        super().__init__(data, **kwargs)
-
-    def process(self):
-
-        self.call_index = self.get_next_bytes(2).hex()
-
-        self.call_module, self.call_function = self.metadata.call_index[self.call_index]
-
-        for arg in self.call_function.args:
-            arg_type_obj = self.process_type(arg.type, metadata=self.metadata)
-
-            self.call_args.append({
-                'name': arg.name,
-                'type': arg.type,
-                'value': arg_type_obj.serialize(),
-                'valueRaw': arg_type_obj.raw_value
-            })
-
-        return {
-            'call_index': self.call_index,
-            'call_function': self.call_function.name,
-            'call_module': self.call_module.name,
-            'call_args': self.call_args
-        }
-
-    def process_encode(self, value):
-        # Check requirements
-        if 'call_index' in value:
-            self.call_index = value['call_index']
-
-        elif 'call_module' in value and 'call_function' in value:
-            # Look up call module from metadata
-            for call_index, (call_module, call_function) in self.metadata.call_index.items():
-
-                if call_module.name == value['call_module'] and call_function.name == value['call_function']:
-                    self.call_index = call_index
-                    self.call_module = call_module
-                    self.call_function = call_function
-                    break
-
-            if not self.call_index:
-                raise ValueError('Specified call module and function not found in metadata')
-
-        elif not self.call_module or not self.call_function:
-            raise ValueError('No call module and function specified')
-
-        data = ScaleBytes(bytearray.fromhex(self.call_index))
-
-        # Encode call params
-        if len(self.call_function.args) > 0:
-            for arg in self.call_function.args:
-                if arg.name not in value['call_args']:
-                    raise ValueError('Parameter \'{}\' not specified'.format(arg.name))
-                else:
-                    param_value = value['call_args'][arg.name]
-
-                    arg_obj = self.get_decoder_class(arg.type, metadata=self.metadata)
-                    data += arg_obj.encode(param_value)
-
-        return data
-
-
 class ProposalPreimage(Struct):
     type_string = '(Vec<u8>, AccountId, BalanceOf, BlockNumber)'
 
@@ -708,6 +688,7 @@ class ProposalPreimage(Struct):
         ("deposit", "BalanceOf"),
         ("blockNumber", "BlockNumber")
     )
+
     def process(self):
 
         result = {}
@@ -715,15 +696,12 @@ class ProposalPreimage(Struct):
             result[key] = self.process_type(data_type, metadata=self.metadata).value
 
         # Replace HexBytes with actual proposal
-        result['proposal'] = Proposal(ScaleBytes(result['proposal']), metadata=self.metadata).decode()
+        result['proposal'] = self.get_decoder_class('Proposal', data=ScaleBytes(result['proposal']),
+                                                    metadata=self.metadata).decode()
 
         return result
 
 
-class Proposal(BoxProposal):
-    type_string = '<T as Trait<I>>::Proposal'
-
-
 class ValidatorPrefs(Struct):
     type_string = '(Compact<Balance>)'
 
@@ -747,31 +725,37 @@ class Linkage(Struct):
 
 class GenericAccountId(H256):
 
-    def __init__(self, data=None, sub_type=None, metadata=None):
+    def __init__(self, data=None, **kwargs):
         self.ss58_address = None
-        super().__init__(data, sub_type, metadata)
+        self.public_key = None
+        super().__init__(data, **kwargs)
 
     def process_encode(self, value):
-        if value[0:2] != '0x' and len(value) == 47:
+        if value[0:2] != '0x':
             from scalecodec.utils.ss58 import ss58_decode
             self.ss58_address = value
             value = '0x{}'.format(ss58_decode(value))
         return super().process_encode(value)
 
+    def serialize(self):
+        return self.ss58_address or self.value
+
+    def process(self):
+        value = self.public_key = super().process()
+
+        if self.runtime_config.ss58_format is not None:
+            try:
+                value = self.ss58_address = ss58_encode(value, ss58_format=self.runtime_config.ss58_format)
+            except ValueError:
+                pass
+
+        return value
+
 
 class GenericAccountIndex(U32):
     pass
 
 
-class KeyValue(Struct):
-    type_string = '(Vec<u8>, Vec<u8>)'
-    type_mapping = (('key', 'Vec<u8>'), ('value', 'Vec<u8>'))
-
-
-class NewAccountOutcome(CompactU32):
-    type_string = 'NewAccountOutcome'
-
-
 class Vec(ScaleType):
 
     def __init__(self, data=None, **kwargs):
@@ -803,21 +787,63 @@ class Vec(ScaleType):
 
         for element in value:
 
-            element_obj = self.get_decoder_class(self.sub_type, metadata=self.metadata)
+            element_obj = self.get_decoder_class(
+                type_string=self.sub_type, metadata=self.metadata, runtime_config=self.runtime_config
+            )
             data += element_obj.encode(element)
 
         return data
 
 
-class BitVec(Vec):
-    # TODO: A BitVec that represents an array of bits. The bits are however stored encoded. The difference between this
-    #  * and a normal Bytes would be that the length prefix indicates the number of bits encoded, not the bytes
-    pass
+class BoundedVec(Vec):
+    def __init__(self, data=None, **kwargs):
+
+        # Rebuild sub_type as last item is the upper bound of elements allowed
+        self.sub_type, self.max_elements = [x.strip() for x in self.sub_type.rsplit(',', 1)]
+
+        super().__init__(data, **kwargs)
+
+
+class BitVec(ScaleType):
+    """
+    A BitVec that represents an array of bits. The bits are however stored encoded. The difference between this
+    and a normal Bytes would be that the length prefix indicates the number of bits encoded, not the bytes
+    """
+    def process(self):
+        length_obj = self.process_type('Compact<u32>')
+
+        total = math.ceil(length_obj.value / 8)
+
+        value_int = int.from_bytes(self.get_next_bytes(total), byteorder='little')
+
+        return '0b' + bin(value_int)[2:].zfill(length_obj.value)
+
+    def process_encode(self, value):
+
+        if type(value) is list:
+            value = sum(v << i for i, v in enumerate(reversed(value)))
+
+        if type(value) is str and value[0:2] == '0b':
+            value = int(value[2:], 2)
+
+        if type(value) is not int:
+            raise ValueError("Provided value is not an int, binary str or a list of booleans")
+
+        if value == 0:
+            return ScaleBytes(b'\x00')
+
+        # encode the length in a compact u32
+        compact_obj = CompactU32()
+        data = compact_obj.encode(value.bit_length())
+
+        byte_length = math.ceil(value.bit_length() / 8)
+
+        return data + value.to_bytes(length=byte_length, byteorder='little')
 
 
 class GenericAddress(ScaleType):
 
-    def __init__(self, data, **kwargs):
+    def __init__(self, data=None, **kwargs):
         self.account_length = None
         self.account_id = None
         self.account_index = None
@@ -914,7 +940,7 @@ class Enum(ScaleType):
     value_list = []
     type_mapping = None
 
-    def __init__(self, data, value_list=None, type_mapping=None, **kwargs):
+    def __init__(self, data=None, value_list=None, type_mapping=None, **kwargs):
 
         self.index = None
 
@@ -932,7 +958,9 @@ class Enum(ScaleType):
         if self.type_mapping:
             try:
                 enum_type_mapping = self.type_mapping[self.index]
-                return self.process_type('Struct', type_mapping=[enum_type_mapping]).value
+                return self.process_type(
+                    type_string='Struct', type_mapping=[enum_type_mapping]
+                ).value
 
             except IndexError:
                 raise ValueError("Index '{}' not present in Enum type mapping".format(self.index))
@@ -959,7 +987,11 @@ class Enum(ScaleType):
                 for idx, (item_key, item_value) in enumerate(self.type_mapping):
                     if item_key == enum_key:
                         self.index = idx
-                        struct_obj = self.get_decoder_class('Struct', type_mapping=[self.type_mapping[self.index]])
+                        struct_obj = self.get_decoder_class(
+                            type_string='Struct',
+                            type_mapping=[self.type_mapping[self.index]],
+                            runtime_config=self.runtime_config
+                        )
                         return ScaleBytes(bytearray([self.index])) + struct_obj.encode(value)
 
                 raise ValueError("Value '{}' not present in type_mapping of this enum".format(enum_key))
@@ -1052,8 +1084,11 @@ class Data(Enum):
                             return ScaleBytes(bytearray(data))
                     else:
 
-                        struct_obj = self.get_decoder_class('Struct', type_mapping=[self.type_mapping[self.index]])
-                        return ScaleBytes(bytearray([self.index])) + struct_obj.encode(value)
+                        struct_obj = self.get_decoder_class(
+                            type_string=self.type_mapping[self.index][1],
+                            runtime_config=self.runtime_config
+                        )
+                        return ScaleBytes(bytearray([self.index + 32])) + struct_obj.encode(enum_value)
 
             raise ValueError("Value '{}' not present in type_mapping of this enum".format(enum_key))
 
@@ -1071,9 +1106,28 @@ class InherentOfflineReport(Null):
     pass
 
 
+class Ticker(ScaleType):
+
+    type_string = 'Ticker'
+
+    def process(self):
+        value = self.get_next_bytes(12)
+        try:
+            return value.decode()
+        except UnicodeDecodeError:
+            return value.hex()
+
+    def process_encode(self, value):
+        if value[0:2] != '0x' and len(value) == 26:
+            raise ValueError(
+                'Value should start with "0x" and should be 12 bytes long')
+        return ScaleBytes(value)
+
+
 class StorageHasher(Enum):
 
-    value_list = ['Blake2_128', 'Blake2_256', 'Blake2_128Concat', 'Twox128', 'Twox256', 'Twox64Concat', 'Identity']
+    value_list = ['Blake2_128', 'Blake2_256', 'Blake2_128Concat',
+                  'Twox128', 'Twox256', 'Twox64Concat', 'Identity']
 
     def is_blake2_128(self):
         return self.index == 0
@@ -1214,6 +1268,7 @@ class GenericCall(ScaleType):
         self.call_function = None
         self.call_args = []
         self.call_module = None
+        self.call_hash = None
 
         super().__init__(data, **kwargs)
 
@@ -1223,21 +1278,27 @@ class GenericCall(ScaleType):
 
         self.call_module, self.call_function = self.metadata.call_index[self.call_index]
 
+        call_bytes = bytes.fromhex(self.call_index)
+
         for arg in self.call_function.args:
             arg_type_obj = self.process_type(arg.type, metadata=self.metadata)
 
+            call_bytes += arg_type_obj.get_used_bytes()
+
             self.call_args.append({
                 'name': arg.name,
                 'type': arg.type,
-                'value': arg_type_obj.serialize(),
-                'valueRaw': arg_type_obj.raw_value
+                'value': arg_type_obj.serialize()
             })
 
+        call_hash = blake2b(call_bytes, digest_size=32).digest()
+
         return {
-            'call_index': self.call_index,
+            'call_index': f'0x{self.call_index}',
             'call_function': self.call_function.name,
             'call_module': self.call_module.name,
-            'call_args': self.call_args
+            'call_args': self.call_args,
+            'call_hash': f'0x{call_hash.hex()}'
         }
 
     def process_encode(self, value):
@@ -1260,7 +1321,9 @@ class GenericCall(ScaleType):
                     break
 
             if not self.call_index:
-                raise ValueError('Specified call module and function not found in metadata')
+                raise MetadataCallFunctionNotFound(
+                    f"Call function '{value['call_module']}.{value['call_function']}' not found in metadata"
+                )
 
         elif not self.call_module or not self.call_function:
             raise ValueError('No call module and function specified')
@@ -1275,15 +1338,48 @@ class GenericCall(ScaleType):
                 else:
                     param_value = value['call_args'][arg.name]
 
-                    arg_obj = self.get_decoder_class(arg.type, metadata=self.metadata)
+                    arg_obj = self.get_decoder_class(
+                        type_string=arg.type, metadata=self.metadata, runtime_config=self.runtime_config
+                    )
                     data += arg_obj.encode(param_value)
         return data
 
 
+class GenericContractExecResult(Enum):
+    def __init__(self, data=None, contract_result_scale_type=None, **kwargs):
+        self.contract_result_scale_type = contract_result_scale_type
+        self.gas_consumed = None
+        self.flags = None
+        self.contract_result_data = None
+        super().__init__(data, **kwargs)
+
+    def process(self):
+        value = super().process()
+        self.process_contract_result()
+        return value
+
+    def process_contract_result(self):
+        if 'success' in self.value:
+            self.gas_consumed = self.value['success']['gas_consumed']
+            self.flags = self.value['success']['flags']
+            self.contract_result_data = self.value['success']['data']
+
+    def process_encode(self, value):
+
+        if self.contract_result_scale_type is None:
+            raise ValueError("Encoding is not possible because 'contract_result_scale_type' is not set")
+
+        if 'success' in value:
+            value = {'Success': value['success']}
+        return super().process_encode(value)
+
+
 class OpaqueCall(Bytes):
 
     def process_encode(self, value):
-        call_obj = self.get_decoder_class('Call', metadata=self.metadata)
+        call_obj = self.get_decoder_class(
+            type_string='Call', metadata=self.metadata, runtime_config=self.runtime_config
+        )
         return super().process_encode(str(call_obj.encode(value)))
 
     def process(self):
@@ -1292,7 +1388,8 @@ class OpaqueCall(Bytes):
             call_obj = self.get_decoder_class(
                 type_string='Call',
                 data=ScaleBytes('0x{}'.format(self.raw_value)),
-                metadata=self.metadata
+                metadata=self.metadata,
+                runtime_config=self.runtime_config
             )
 
             return call_obj.process()
@@ -1312,7 +1409,7 @@ class MultiAccountId(GenericAccountId):
                 account = '0x{}'.format(ss58_decode(account))
             account_ids.append(account)
 
-        account_list_cls = cls.get_decoder_class('Vec<AccountId>')
+        account_list_cls = cls.get_decoder_class('Vec<AccountId>', runtime_config=cls.runtime_config)
         account_list_data = account_list_cls.encode(sorted(account_ids))
         threshold_data = cls.get_decoder_class("u16").encode(threshold)
 
@@ -1320,7 +1417,7 @@ class MultiAccountId(GenericAccountId):
             b"modlpy/utilisuba" + bytes(account_list_data.data) + bytes(threshold_data.data), digest_size=32
         ).digest().hex())
 
-        multi_account_obj = cls()
+        multi_account_obj = cls(runtime_config=cls.runtime_config)
         multi_account_obj.encode(multi_account_id)
 
         return multi_account_obj
@@ -1362,43 +1459,125 @@ class FixedLengthArray(ScaleType):
                 raise ValueError('Given value is not a list')
 
             for element_value in value:
-                element_obj = self.get_decoder_class(self.sub_type, metadata=self.metadata)
+                element_obj = self.get_decoder_class(
+                    type_string=self.sub_type, metadata=self.metadata, runtime_config=self.runtime_config
+                )
                 data += element_obj.encode(element_value)
 
             return data
 
-class Ticker(ScaleType):
-
-    type_string = 'Ticker'
-
-    def process(self):
-        value = self.get_next_bytes(12)
-        try:
-            return value.decode()
-        except UnicodeDecodeError:
-            return value.hex()
-
-    def process_encode(self, value):
-        if value[0:2] != '0x' and len(value) == 26:
-            raise ValueError('Value should start with "0x" and should be 12 bytes long')
-        return ScaleBytes(value)
-
 
 class GenericMultiAddress(Enum):
     type_mapping = [
         ["Id", "AccountId"],
         ["Index", "Compact<AccountIndex>"],
-        ["Raw", "Bytes"],
+        ["Raw", "HexBytes"],
         ["Address32", "H256"],
         ["Address20", "H160"],
       ]
 
+    def __init__(self, data, **kwargs):
+        self.account_length = None
+        self.account_id = None
+        self.account_index = None
+        self.account_idx = None
+        super().__init__(data, **kwargs)
+
+    def process(self):
+        value = super().process()
+        self.account_length = self.index
+        if self.index == 0:
+            value = list(value.values())[0]
+
+            if is_valid_ss58_address(value):
+                self.account_id = ss58_decode(value)
+            elif value[0:2] == '0x':
+                self.account_id = value[2:]
+
+            return value
+        elif self.index == 1:
+            self.account_index = list(value.values())[0]
+            return self.account_index
+        else:
+            # Todo cap HexBytes / zero pad to 32 bytes to keep compatibility with AccountId
+            account_id = list(value.values())[0]
+            self.account_id = account_id[2:66].ljust(64, '0')
+            return value
+
     def process_encode(self, value):
 
-        if type(value) == str:
-            if len(value) <= 8:
+        if type(value) is int:
+            # Implied decoded AccountIndex
+            value = {"Index": value}
+
+        elif type(value) is str:
+            if len(value) <= 8 and value[0:2] != '0x':
+                # Implied raw AccountIndex
                 value = {"Index": ss58_decode_account_index(value)}
-            else:
+            elif is_valid_ss58_address(value):
+                # Implied SS58 encoded AccountId
+                value = {"Id": f'0x{ss58_decode(value)}'}
+            elif len(value) == 66 and value[0:2] == '0x':
+                # Implied raw AccountId
                 value = {"Id": value}
+            elif len(value) == 42:
+                # Implied raw Address20
+                value = {"Address20": value}
+            else:
+                raise NotImplementedError("Address type not yet supported")
 
         return super().process_encode(value)
+
+
+class Map(ScaleType):
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.map_key, self.map_value = [x.strip() for x in self.sub_type.split(',')]
+
+    def process(self):
+
+        element_count = self.process_type('Compact<u32>').value
+
+        result = []
+        for _ in range(0, element_count):
+            key_value = self.process_type(self.map_key, metadata=self.metadata).value
+            result.append((key_value, self.process_type(self.map_value, metadata=self.metadata).value))
+
+        return result
+
+    def process_encode(self, value):
+
+        if type(value) is not list:
+            raise ValueError("value should be a list of tuples e.g.: [('1', 2), ('23', 24), ('28', 30), ('45', 80)]")
+
+        element_count_compact = CompactU32()
+
+        element_count_compact.encode(len(value))
+
+        data = element_count_compact.data
+
+        for item_key, item_value in value:
+            key_obj = self.get_decoder_class(
+                type_string=self.map_key, metadata=self.metadata, runtime_config=self.runtime_config
+            )
+            data += key_obj.encode(item_key)
+
+            value_obj = self.get_decoder_class(
+                type_string=self.map_value, metadata=self.metadata, runtime_config=self.runtime_config
+            )
+            data += value_obj.encode(item_value)
+
+        return data
+
+
+class HashMap(Map):
+    pass
+
+
+class BTreeMap(Map):
+    pass
+
+
+class BTreeSet(Vec):
+    pass
